package top.dixxe.bombezhka.listeners;

import org.bukkit.Bukkit;
import org.bukkit.event.Listener;
import org.bukkit.plugin.java.JavaPlugin;
import top.dixxe.bombezhka.events.TelegramMessageEvent;

import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

public class TelegramMessageListener implements Listener {
    private final JavaPlugin plugin;
    private final HttpClient httpClient;
    private final String botToken;
    private final long chatId;
    private int lastUpdateId = 0;

    public TelegramMessageListener(JavaPlugin plugin, String botToken, long chatId) {
        this.plugin = plugin;
        this.botToken = botToken;
        this.chatId = chatId;
        this.httpClient = HttpClient.newHttpClient();
    }

    public void startPolling() {
        // Poll every 20 seconds (adjust as needed)
        plugin.getServer().getScheduler().runTaskTimerAsynchronously(plugin, this::pollTelegram, 0L, 20 * 20L);
    }

    private void pollTelegram() {
        try {
            String url = String.format("https://api.telegram.org/bot%s/getUpdates?offset=%d&timeout=30", botToken, lastUpdateId + 1);
            HttpRequest request = HttpRequest.newBuilder().uri(URI.create(url)).build();

            HttpResponse<String> response = httpClient.send(request, HttpResponse.BodyHandlers.ofString());
            if (response.statusCode() == 200) {
                parseUpdates(response.body());
            } else {
                if (response.statusCode() == 409) { return; }
                plugin.getLogger().warning("Telegram API error: " + response.statusCode());
            }
        } catch (Exception e) {
            plugin.getLogger().warning("Polling error: " + e.getMessage());
        }
    }

    private void parseUpdates(String json) {
        try {
            com.google.gson.JsonObject root = com.google.gson.JsonParser.parseString(json).getAsJsonObject();
            if (root.get("ok").getAsBoolean()) {
                com.google.gson.JsonArray updates = root.getAsJsonArray("result");
                for (com.google.gson.JsonElement element : updates) {
                    com.google.gson.JsonObject update = element.getAsJsonObject();
                    int updateId = update.get("update_id").getAsInt();
                    lastUpdateId = updateId;

                    // Changed: Check for "message" instead of "channel_post"
                    if (update.has("message")) {
                        com.google.gson.JsonObject message = update.getAsJsonObject("message");
                        com.google.gson.JsonObject chat = message.getAsJsonObject("chat");

                        // Get chat type and ID
                        String chatType = chat.get("type").getAsString();
                        long messageChatId = chat.get("id").getAsLong();

                        // Check if it's a group/supergroup and matches configured chat ID
                        if ((chatType.equals("group") || chatType.equals("supergroup")) &&
                                messageChatId == chatId &&
                                message.has("text")) {

                            String text = message.get("text").getAsString();

                            plugin.getServer().getScheduler().runTask(plugin, () -> {
                                Bukkit.getPluginManager().callEvent(new TelegramMessageEvent(text));
                            });
                        }
                    }
                }
            }
        } catch (Exception e) {
            plugin.getLogger().warning("JSON parsing error: " + e.getMessage());
        }
    }

}